"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var async = require("async");
var fiberit_1 = require("./fiberit");
var R = require("ramda");
var Parallel = /** @class */ (function () {
    function Parallel() {
    }
    /**
     * Executes the syncFunctionOrFunctionWithFiber in parallel over the array values.
     * Accepts a function that is either synchronous or uses coroutines internally (Fiberit).
     * Hint: when there's an error like "callback already called" is that
     * there's a runtime error not managed from async
     * @param array A[]
     * @param syncFunctionOrFunctionWithFiber (A)=>B
     * @returns B[]
     * @throws Error
     */
    Parallel.map = function (array, syncFunctionOrFunctionWithFiber) {
        if (array.length == 0)
            return [];
        var res = this.mapAsync(array, syncFunctionOrFunctionWithFiber);
        var errors = res
            .filter(function (elem) { return elem instanceof Error; });
        var msg = errors.reduce(function (acc, err, index) {
            acc[index] = { msg: err.message, stack: err.stack };
            return acc;
        }, {});
        if (msg[0])
            throw new Error(JSON.stringify(msg));
        return res;
    };
    Parallel.zipMap = function (functs, values) {
        var zipped = R.zip(functs, values);
        return Parallel.map(zipped, function (_a) {
            var fun = _a[0], val = _a[1];
            return fun(val);
        });
    };
    Parallel.zipMapWith = function (functs, values) {
        var zipped = R.zip(functs, values);
        return Parallel.map(zipped, function (_a) {
            var fun = _a[0], val = _a[1];
            return fun.apply(null, val);
        });
    };
    Parallel.withData = function (array) {
        return function (action) { return Parallel.map(array, action); };
    };
    Parallel.pool = function (size, data, mapper) {
        var dataSplittedBySize = R.splitEvery(size, data);
        var results = dataSplittedBySize
            .map(function (pairOfData) { return Parallel.map(pairOfData, mapper); });
        return R.flatten(results);
    };
    Parallel.poolWith = function (size, data, mapper) {
        var dataSplittedBySize = R.splitEvery(size, data);
        var resultsSplittedBySize = dataSplittedBySize
            .map(function (data) { return Parallel.map(data, function (input) { return mapper.apply(null, input); }); });
        return R.flatten(resultsSplittedBySize);
    };
    Parallel.mapAsync = function (array, fn) {
        return fiberit_1.Fiberit.for(async.map, array, function (input, cb) {
            return fiberit_1.Fiberit.launchFiber(function () {
                try {
                    var res = fn.call(fn, input);
                    return cb(null, res);
                }
                catch (e) {
                    return cb(e);
                }
            });
        });
    };
    return Parallel;
}());
exports.Parallel = Parallel;
//# sourceMappingURL=parallel.js.map